
import os
import re

from bzrlib.errors import BzrError
from ConfigParser import NoOptionError, RawConfigParser


class Repo(object):

    def __init__(self):
        self.read_access = []
        self.write_access = []

    def allow_read(self, users=[]):
        for user in users:
            if not user in self.read_access:
                self.read_access.append(user)

    def allow_write(self, users=[]):
        self.allow_read(users)
        for user in users:
            if not user in self.write_access:
                self.write_access.append(user)

    def access_read(self, user):
        return user in self.read_access

    def access_write(self, user):
        return user in self.write_access


def parse_config(config):
    path = None
    repos = {}

    parser = RawConfigParser()
    parser.read(config)

    for section in parser.sections():
        if section == 'main':
            try:
                path = parser.get(section, 'path')
            except NoOptionError:
                raise BzrError('acl-serve: invalid config: missing root path')
            if not os.access(path, os.W_OK):
                raise BzrError('acl-serve: cannot write root path')
        else:
            if not section in repos:
                repos[section] = Repo()
            try:
                R = parser.get(section, 'R')
                if R:
                    repos[section].allow_read(R.split())
            except NoOptionError:
                pass
            try:
                RW = parser.get(section, 'RW')
                if RW:
                    repos[section].allow_write(RW.split())
            except NoOptionError:
                pass

    if not path:
        raise BzrError('acl-serve: invalid config: missing root path')

    return path, repos


class ACL(object):

    def __init__(self, config, user):
        self.user = user
        self.root, self.repos = parse_config(config)

    def root_path(self):
        return self.root

    def access_read(self, path):
        access = False
        for repo in sorted(self.repos, reverse=True):
            if repo == path or path.startswith(repo):
                access = self.repos[repo].access_read(self.user)
            if access:
                break
        return access

    def access_write(self, path):
        access = False
        for repo in sorted(self.repos, reverse=True):
            if repo == path or path.startswith(repo):
                access = self.repos[repo].access_write(self.user)
            if access:
                break
        return access
