
from bzrlib.errors import BzrError
from bzrlib.smart.server import BzrServerFactory
from bzrlib.transport.pathfilter import (
    PathFilteringServer,
    PathFilteringTransport
)


class ACLServeServer(PathFilteringServer):

    def __init__(self, backing_transport, filter_func, acl):
        PathFilteringServer.__init__(self, backing_transport, filter_func)
        self.acl = acl

    def _factory(self, url):
        return ACLServeTransport(self, url, self.acl)


class ACLServeTransport(PathFilteringTransport):

    def __init__(self, server, base, acl):
        PathFilteringTransport.__init__(self, server, base)
        self.acl = acl

    def clone(self, relpath):
        return self.__class__(self.server, self.abspath(relpath), self.acl)

    read_methods = ['get', 'has', 'list_dir', 'stat']
    write_methods = ['append_file', 'delete', 'delete_tree', 'mkdir',
                     'open_write_stream', 'put_file', 'rename', 'rmdir']

    def _call(self, methodname, relpath, *args):
        if (methodname not in self.read_methods and
            methodname not in self.write_methods):
                raise BzrError('acl-serve: access denied')

        if methodname in self.read_methods:
            if not self.acl.access_read(self.base_path):
                raise BzrError('acl-serve: access denied')
        elif methodname in self.write_methods:
            if not self.acl.access_write(self.base_path):
                raise BzrError('acl-serve: access denied')

        return PathFilteringTransport._call(self, methodname, relpath, *args)


class ACLServeServerFactory(BzrServerFactory):

    def __init__(self, acl):
        BzrServerFactory.__init__(self)
        self.acl = acl

    def _make_expand_userdirs_filter(self, transport):
        return ACLServeServer(transport, self._expand_userdirs, self.acl)


def serve(acl, transport, host=None, port=None, inet=False, timeout=None):
    server = ACLServeServerFactory(acl)
    try:
        server.set_up(transport, host, port, inet)
        server.smart_server.serve()
    finally:
        server.tear_down()
